@@ -5,7 +5,7 @@
  * Python module that implements the libparted functionality via Python
  * classes and other high level language features.
  *
- * Copyright (C) 2007-2015 Red Hat, Inc.
+ * Copyright (C) 2007-2020 Red Hat, Inc.
  *
  * This copyrighted material is made available to anyone wishing to use,
  * modify, copy, or redistribute it subject to the terms and conditions of
@@ -42,19 +42,29 @@
 #include "pytimer.h"
 #include "pyunit.h"
 
-#if PED_DISK_LAST_FLAG < 2
-#define PED_DISK_GPT_PMBR_BOOT 2
-#endif
-
-#if PED_PARTITION_LAST_FLAG < 15
-#define PED_PARTITION_LEGACY_BOOT 15
-#endif
-
 char *partedExnMessage = NULL;
 unsigned int partedExnRaised = 0;
 
 PyObject *exn_handler = NULL;
 
+/* custom exceptions for _ped */
+/* These are declared in exceptions.h for use elsewhere. */
+PyObject *AlignmentException;
+PyObject *CreateException;
+PyObject *ConstraintException;
+PyObject *DeviceException;
+PyObject *DiskException;
+PyObject *DiskLabelException;
+PyObject *FileSystemException;
+PyObject *GeometryException;
+PyObject *IOException;
+PyObject *NotNeededException;
+PyObject *PartedException;
+PyObject *PartitionException;
+PyObject *TimerException;
+PyObject *UnknownDeviceException;
+PyObject *UnknownTypeException;
+
 /* Docs strings are broken out of the module structure here to be at least a
  * little bit readable.
  */
@@ -415,43 +425,46 @@ PyObject *py_pyparted_version(PyObject *s, PyObject *args) {
  * py_ped_register_exn_handler function.
  */
 static PedExceptionOption partedExnHandler(PedException *e) {
+    PedExceptionOption ret;
+
     switch (e->type) {
-        /* Raise yes/no exceptions so the caller can deal with them,
+        /* Raise yes/no/fix exceptions so the caller can deal with them,
          * otherwise ignore */
         case PED_EXCEPTION_INFORMATION:
         case PED_EXCEPTION_WARNING:
             if (e->options == PED_EXCEPTION_YES_NO) {
-                partedExnRaised = 1;
-                partedExnMessage = strdup(e->message);
-
-                if (partedExnMessage == NULL)
-                    PyErr_NoMemory();
-                else if (exn_handler && PyCallable_Check(exn_handler)) {
-                    PyObject *args, *retval;
-
-                    args = PyTuple_New(3);
-                    PyTuple_SetItem(args, 0, PyLong_FromLong(e->type));
-                    PyTuple_SetItem(args, 1, PyLong_FromLong(e->options));
-                    PyTuple_SetItem(args, 2, PyUnicode_FromString(e->message));
-
-                    retval = PyObject_CallObject(exn_handler, args);
-                    Py_DECREF(args);
-                    if (retval != NULL && (PyLong_AsLong(retval) == PED_EXCEPTION_UNHANDLED || (PyLong_AsLong(retval) & e->options) > 0))
-                        return PyLong_AsLong(retval);
-                    else
-                        /* Call failed, use the default value. */
-                        return PED_EXCEPTION_NO;
-                }
-                else {
-                    /* If no exception handling function was registered to
-                     * tell us what to do, return "no" for any yes/no
-                     * questions to prevent any potential disk destruction.
-                     */
-                    return PED_EXCEPTION_NO;
-                }
+                ret = PED_EXCEPTION_NO;
+            } else if (e->options & PED_EXCEPTION_IGNORE) {
+                ret = PED_EXCEPTION_IGNORE;
             } else {
                 partedExnRaised = 0;
-                return PED_EXCEPTION_IGNORE;
+                return PED_EXCEPTION_UNHANDLED;
+            }
+
+            partedExnRaised = 1;
+            partedExnMessage = strdup(e->message);
+
+            if (partedExnMessage == NULL) {
+                PyErr_NoMemory();
+            } else if (exn_handler && PyCallable_Check(exn_handler)) {
+                PyObject *args, *retval;
+
+                args = PyTuple_New(3);
+                PyTuple_SetItem(args, 0, PyLong_FromLong(e->type));
+                PyTuple_SetItem(args, 1, PyLong_FromLong(e->options));
+                PyTuple_SetItem(args, 2, PyUnicode_FromString(e->message));
+
+                retval = PyObject_CallObject(exn_handler, args);
+                Py_DECREF(args);
+                if (retval != NULL && (PyLong_AsLong(retval) == PED_EXCEPTION_UNHANDLED || (PyLong_AsLong(retval) & e->options) > 0)) {
+                    return PyLong_AsLong(retval);
+                }
+            } else {
+                /* If no exception handling function was registered to
+                 * tell us what to do, return "no" for any yes/no
+                 * questions to prevent any potential disk destruction.
+                 */
+                return ret;
             }
 
         /* Set global flags so parted module methods can raise specific
@@ -634,9 +647,23 @@ MOD_INIT(_ped) {
     PyModule_AddIntConstant(m, "PARTITION_BIOS_GRUB", PED_PARTITION_BIOS_GRUB);
     PyModule_AddIntConstant(m, "PARTITION_DIAG", PED_PARTITION_DIAG);
     PyModule_AddIntConstant(m, "PARTITION_LEGACY_BOOT", PED_PARTITION_LEGACY_BOOT);
-    PyModule_AddIntConstant(m, "PARTITION_MSFT_DATA", PED_PARTITION_MSFT_DATA);
-    PyModule_AddIntConstant(m, "PARTITION_IRST", PED_PARTITION_IRST);
-    PyModule_AddIntConstant(m, "PARTITION_ESP", PED_PARTITION_ESP);
+
+    /* NOTE: You cannot evaluate the enum PED_PARTITION_* values using the
+     * preprocessor. DO NOT use #if or #ifdef with them.
+     *
+     * Conditionally add constants, based on PED_PARTITION_LAST_FLAG, and what
+     * we know about parted/disk.h
+     */
+    if (PED_PARTITION_LAST_FLAG > 15)
+        PyModule_AddIntConstant(m, "PARTITION_MSFT_DATA", PED_PARTITION_MSFT_DATA);
+    if (PED_PARTITION_LAST_FLAG > 16)
+        PyModule_AddIntConstant(m, "PARTITION_IRST", PED_PARTITION_IRST);
+    if (PED_PARTITION_LAST_FLAG > 17)
+        PyModule_AddIntConstant(m, "PARTITION_ESP", PED_PARTITION_ESP);
+    if (PED_PARTITION_LAST_FLAG > 18)
+        PyModule_AddIntConstant(m, "PARTITION_CHROMEOS_KERNEL", PED_PARTITION_CHROMEOS_KERNEL);
+    if (PED_PARTITION_LAST_FLAG > 19)
+        PyModule_AddIntConstant(m, "PARTITION_BLS_BOOT", PED_PARTITION_BLS_BOOT);
 
     PyModule_AddIntConstant(m, "DISK_CYLINDER_ALIGNMENT", PED_DISK_CYLINDER_ALIGNMENT);
     PyModule_AddIntConstant(m, "DISK_GPT_PMBR_BOOT", PED_DISK_GPT_PMBR_BOOT);
